﻿// charting
// http://www.chartjs.org/
//
//

/// <reference path="//cdnjs.cloudflare.com/ajax/libs/toastr.js/2.1.0/js/toastr.min.js" />
function ODSClient(options) {
    this.App = {
        APIPath: '/api/oct2014/'
    };
    this.Temp = {};
    this.Forms = {};
    this.Controls = {
        AddOptionsToSelect: function (id, obj, displaymember, valuemember) {
            var select = document.getElementById(id);
            if (Object.prototype.toString.call(obj) === '[object Array]') {
                for (var i = 0, l = obj.length; i < l; i++) {
                    var prop = obj[i];
                    var option = document.createElement('option');
                    option.value = prop[valuemember];
                    option.appendChild(document.createTextNode(prop[displaymember]));
                    select.appendChild(option);
                }
            }
            else {
                for (var p in obj) {
                    var prop = obj[p];
                    var option = document.createElement('option');
                    option.value = prop[valuemember];
                    option.appendChild(document.createTextNode(prop[displaymember]));
                    select.appendChild(option);
                }
            }
            
        },
        CompleteDocumentPropertyForm: function (id, documentProperty) {

        },
        Checkbox: function (options) {
            var cbWrapper = document.createElement('div'),
            checkbox = document.createElement('input'),
            lbl = document.createElement('label');
            cbWrapper.className = 'checkbox checkbox-primary';
            checkbox.id = options.id;
            checkbox.type = 'checkbox';
            if (options.disabled == true) {
                checkbox.disabled = true;
            }
            lbl.setAttribute('for', checkbox.id);
            lbl.appendChild(document.createTextNode(options.text));
            cbWrapper.appendChild(checkbox);
            cbWrapper.appendChild(lbl);
            document.getElementById(options.target).appendChild(cbWrapper);
        }
    };
    this.HttpStatusCodes = {
        '100': 'Continue',
        '101': 'Switching Protocols',
        '103': 'Checkpoint',
        '200': 'OK',
        '201': 'Created',
        '202': 'Accepted',
        '203': 'Non-Authoritative Information',
        '204': 'No Content',
        '205': 'Reset Content',
        '206': 'Partial Content',
        '300': 'Multiple Choices',
        '301': 'Moved Permanently',
        '302': 'Found',
        '303': 'See Other',
        '304': 'Not Modified',
        '306': 'Switch Proxy',
        '307': 'Temporary Redirect',
        '308': 'Resume Incomplete',
        '400': 'Bad Request',
        '401': 'Unauthorized',
        '402': 'Payment Required',
        '403': 'Forbidden',
        '404': 'Not Found',
        '405': 'Method Not Allowed',
        '406': 'Not Acceptable',
        '407': 'Proxy Authentication Required',
        '408': 'Request Timeout',
        '409': 'Conflict',
        '410': 'Gone',
        '411': 'Length Required',
        '412': 'Precondition Failed',
        '413': 'Request Entity Too Large',
        '414': 'Request-URI Too Long',
        '415': 'Unsupported Media Type',
        '416': 'Requested Range Not Satisfiable',
        '417': 'Expectation Failed',
        '500': 'Internal Server Error',
        '501': 'Not Implemented',
        '502': 'Bad Gateway',
        '503': 'Service Unavailable',
        '504': 'Gateway Timeout',
        '505': 'HTTP Version Not Supported',
        '511': 'Network Authentication Required'
    };
    this.AcceptInvitation = function (pwd, iid, cb, fb) {
        var client = this;
        var hdrs = {
            'acct-pwd': pwd
        };
        $.ajax({
            url: client.App.APIPath + 'useraccounts/invited/' + iid,
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: hdrs
        })
        .done(function (aKey) {
            client.App.AuthenticationKey = aKey;
            client.App.APIHeaders = {
                'x-auth-sid': aKey.SolutionId,
                'x-auth-uid': aKey.UserAccountId,
                'x-auth-ses': aKey.SessionKey,
                'x-auth-tok': aKey.Token
            };
            cb(aKey);
        })
        .fail(function (rsp) { var r = rsp; client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, function () { fb(r); }); });
    };
    this.AuthenticatePK = function (pk, cb, fb) {
        var client = this;
        var hdrs = {
            'x-auth-pky': pk
        };
        if (localStorage.WorkstationId != null) {
            hdrs['x-auth-wid'] = localStorage.WorkstationId;
        }
        $.ajax({
            url: client.App.APIPath + 'authenticationkeys/public',
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: hdrs
        })
        .done(function (aKey) {
            client.App.AuthenticationKey = aKey;
            client.App.APIHeaders = {
                'x-auth-sid': aKey.SolutionId,
                'x-auth-uid': aKey.UserAccountId,
                'x-auth-ses': aKey.SessionKey,
                'x-auth-tok': aKey.Token
            };
            cb(aKey);
        })
        .fail(function (rsp) { var r = rsp; client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, function () { fb(r); }); });
    };
    this.Authenticate = function (eml, pwd, cb, fb) {
        var client = this;
        var hdrs = {
            'acct-eml': eml,
            'acct-pwd': pwd
        };
        if (localStorage.WorkstationId != null) {
            hdrs['x-auth-wid'] = localStorage.WorkstationId;
        }
        $.ajax({
            url: client.App.APIPath + 'authenticationkeys',
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: hdrs
        })
        .done(function (aKey) {
            client.App.AuthenticationKey = aKey;
            client.App.APIHeaders = {
                'x-auth-sid': aKey.SolutionId,
                'x-auth-uid': aKey.UserAccountId,
                'x-auth-ses': aKey.SessionKey,
                'x-auth-tok': aKey.Token
            };
            cb(aKey);
        })
        .fail(function (rsp) { var r = rsp; client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, function () { fb(r); }); });
    };
    this.SignOut = function (cb, fb) {
        this.ResetApp(function () { window.location = 'login'; });
    };
    this.ListSolutions = function (cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'solutions/list',
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (solutions) { cb(solutions); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.GetSolution = function (cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'solutions',
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done( function (solution) { cb(solution); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.SaveSolution = function (sol, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'solutions',
            type: 'POST',
            contentType: 'application/json;charset=utf-8',
            data: JSON.stringify(sol),
            headers: client.App.APIHeaders
        })
        .done(function (solution) { cb(solution); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.GetSchema = function (cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'solutions/schema',
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (schema) { cb(schema); })
        .fail(function (rsp) {
            if (rsp.status == 401) {
                client.OpenDialog('loginform');
            }
            else {
                client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb);
            }
        });
    };
    this.SaveSchema = function (sch, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'solutions/schema',
            type: 'POST',
            contentType: 'application/json;charset=utf-8',
            data: JSON.stringify(sch),
            headers: client.App.APIHeaders
        })
        .done(function (schema) { cb(schema); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.ListUserAccounts = function (cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'useraccounts',
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(
            function (userAccounts) {
                cb(userAccounts);
            })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.GetUserAccount = function (uid, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'useraccounts/' + uid,
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (userAccount) {cb(userAccount); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.SaveUserAccount = function (uac, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'useraccounts',
            type: 'PUT',
            contentType: 'application/json;charset=utf-8',
            data: JSON.stringify(uac),
            headers: client.App.APIHeaders
        })
        .done(function (uac) { cb(uac); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.InviteUser = function (uac, tid, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'useraccounts/invite/' + tid,
            type: 'POST',
            contentType: 'application/json;charset=utf-8',
            data: JSON.stringify(uac),
            headers: client.App.APIHeaders
        })
        .done(function (uac) { cb(uac); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.InviteUserCustomSubject = function (uac, tid, sub, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'useraccounts/invite/' + tid + '/' + sub,
            type: 'POST',
            contentType: 'application/json;charset=utf-8',
            data: JSON.stringify(uac),
            headers: client.App.APIHeaders
        })
        .done(function (uac) { cb(uac); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.SendRecoveryEmail = function (eml, tid, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'useraccounts/recover/' + tid,
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: { 'acct-eml': eml }
        })
        .done(function (rsp) { cb(rsp); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.SaveUserAccountWithTemplateEmail = function (uac, eid, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'useraccounts/' + eid,
            type: 'POST',
            contentType: 'application/json;charset=utf-8',
            data: JSON.stringify(uac),
            headers: client.App.APIHeaders
        })
        .done(function (uac) { cb(uac); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.SetWorkstationId = function (uid, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'useraccounts/' + uid + '/setworkstation',
            type: 'PUT',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (wid) {
            localStorage.WorkstationId = wid;
            cb(wid);
        })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    }
    this.RemoveWorkstationId = function (uid, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'useraccounts/' + uid + '/remworkstation',
            type: 'PUT',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeader
        })
        .done(function (rsp) {
            localStorage.WorkstationId = null;
            cb(rsp);
        })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.ChangeUserPwd = function (opw, npw, cb, fb) {
        var client = this;
        var customHeaders = client.App.APIHeaders;
        customHeaders['x-auth-old'] = opw;
        customHeaders['x-auth-npw'] = npw;
        $.ajax({
            url: client.App.APIPath + 'authenticationkeys/changepassword',
            type: 'PUT',
            contentType: 'application/json;charset=utf-8',
            headers: customHeaders
        })
        .done(function (rsp) { cb(rsp); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.ResetUser = function (eml, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'authenticationkeys/' + eml + '/reset',
            type: 'PUT',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (rsp) { cb(rsp); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.ListDocuments = function (cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'useraccounts',
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (uac) { cb(uac); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.GetDocument = function (did, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'documents/' + did,
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (doc) { cb(doc); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.SaveDocument = function (doc, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'documents',
            type: 'POST',
            contentType: 'application/json;charset=utf-8',
            data: JSON.stringify(doc),
            headers: client.App.APIHeaders
        })
        .done(function (d) { cb(d); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.RenderDocumentFile = function (did, fid, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'documents/' + did + '/' + fid + '/render',
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (rsp) { cb(rsp); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.SnapshotDocumentFile = function (did, fid, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'documents/' + did + '/' + fid + '/snapshot',
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (rsp) { cb(rsp); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.OCRDocument = function (did, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'documents/' + did + '/ocr',
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (rsp) { cb(rsp); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.IndexDocument = function (did, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'documents/' + did + '/index',
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (rsp) { cb(rsp); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.SendEmail = function (to, from, subject, body, cb, fb) {
        var client = this,
            emailMessage = {
                To: to,
                From: from,
                Subject: subject,
                Body: body
            };
        $.ajax({
            url: client.App.APIPath + 'utilities/sendmail',
            type: 'POST',
            contentType: 'application/json;charset=utf-8',
            data: JSON.stringify(emailMessage),
            headers: client.App.APIHeaders
        })
        .done(function (rsp) { cb(rsp); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    }
    this.Search = function (query, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'documents/search',
            type: 'POST',
            contentType: 'application/json;charset=utf-8',
            data: JSON.stringify(query),
            headers: client.App.APIHeaders
        })
        .done(function (documents) { cb(documents); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.PageSearch = function (query, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'documents/pagesearch',
            type: 'POST',
            contentType: 'application/json;charset=utf-8',
            data: JSON.stringify(query),
            headers: client.App.APIHeaders
        })
        .done(function (documents) { cb(documents); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.SaveForm = function (cb, fb) {
        // use this function to save document form to service
    };
    this.OpenForm = function (formId, target, cb, fb) {
        $.get(formId, function (formContent) {
            $('#' + target).html(formContent);
        });
    };
    this.CloseForm = function (target, cb, fb) {
        $('#' + target).html('');
        cb();
    };
    this.OpenDialog = function (formId, cb, fb) {
        $.get(formId, function (formContent) {
            $('body').append(formContent);
            if (formId == 'loginform') {
                $('#' + formId).modal({ backdrop: 'static' }).modal('show').on('hidden.bs.modal', function () { $('#' + formId).remove(); });
            }
            else if (formId == 'propertyform') {
                $('#' + formId).modal({ backdrop: false }).modal('show').on('hidden.bs.modal', function () { $('#' + formId).remove(); });
            }
            else {
                $('#' + formId).modal('show').on('hidden.bs.modal', function () { $('#' + formId).remove(); });
            }
        });
    };
    this.CloseDialog = function (formId, cb, fb) {
        $('#' + formId).modal('hide');
        if (cb != null) {
            cb();
        }
    };
    this.SaveApp = function (cb, fb) {
        localStorage.App = JSON.stringify(this.App);
        cb();
    };
    this.LoadApp = function (cb, fb) {
        if (localStorage.App != null) {
            this.App = JSON.parse(localStorage.App);
        }
        cb();
    };
    this.ResetApp = function (cb, fb) {
        this.App = { APIPath: '/api/oct2014/' };
        this.SaveApp(cb);
    };
    // toastr
    this.CongratulateUser = function (hdr, msg, cb, fb) {
        toastr.success(msg, hdr);
        if (cb != null) {
            cb();
        }
    };
    this.WarnUser = function (hdr, msg, cb, fb) {
        toastr.warning(msg, hdr);
        if (cb != null) {
            cb();
        }
    };
    this.AlertUser = function (hdr, msg, cb, fb) {
        toastr.error(msg, hdr);
        if (cb != null) {
            cb();
        }
    };
    this.NotifyUser = function (hdr, msg, cb, fb) {
        toastr.info(msg, hdr);
        if (cb != null) {
            cb();
        }
    };
    this.KeyGen = function (did, fid, exp, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'utilities/keygen/' + did + '/' + fid + '/' + exp,
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (key) { cb(key); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    // utilities
    this.StringToDate = function (dateAsString) {
        var dateParts;
        if (dateAsString.indexOf('/') > -1) {
            dateParts = dateAsString.split('/');
            if (dateParts.length == 3) {
                return new Date(this.Pad(dateParts[2], 4), this.Pad(dateParts[0] - 1, 2), this.Pad(dateParts[1], 2));
            }
            else {
                ods.AlertUser('Date Format Exception', 'The date format you entered is not valid.');
            }
        }
        else if (dateAsString.indexOf('-') > -1) {
            dateParts = dateAsString.split('-');
            if (dateParts.length == 3) {
                return new Date(this.Pad(dateParts[2], 4), this.Pad(dateParts[0] - 1, 2), this.Pad(dateParts[1], 2));
            }
            else {
                ods.AlertUser('Date Format Exception', 'The date format you entered is not valid.');
            }
        }
        else if (dateAsString.indexOf('.') > -1) {
            dateParts = dateAsString.split('.');
            if (dateParts.length == 3) {
                return new Date(this.Pad(dateParts[2], 4), this.Pad(dateParts[0] - 1, 2), this.Pad(dateParts[1], 2));
            }
            else {
                ods.AlertUser('Date Format Exception', 'The date format you entered is not valid.');
            }
        }
        else {
            ods.AlertUser('Date Format Exception', 'The date format you entered is not valid.');
        }
    };
    this.GetTimestamp = function (d) { // Date d
        // Get date's subparts as variables:
        var y = d.getUTCFullYear(), //   4 digits
            M = d.getUTCMonth() + 1,// 1-2 digits
            D = d.getUTCDate(),     // 1-2 digits
            h = d.getUTCHours(),    // 1-2 digits
            m = d.getUTCMinutes(),  // 1-2 digits
            s = d.getUTCSeconds();  // 1-2 digits

        // 2. Adjust lengths to be right:
        function l2(x) { // for months, days, hours, seconds. Force length=2 digits.
            x = x + ''; // stingify
            if (x.length === 1) { return '0' + x; }
            return x;
        }

        // Concatenate to YYYYMMMDDDhhmmssxxx format:
        return y + l2(M) + l2(D) + l2(h) + l2(m) + l2(s);
    };
    this.GetDate = function (ts) {
        var date = new Date(ts.replace(
            /^(\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$/,
            '$4:$5:$6 $2/$3/$1'
        ));
        return date;
    };
    this.FormatDate = function (date) {
        var mm = date.getMonth() + 1;
        var dd = date.getDate();
        var yyyy = date.getFullYear();
        var formatted = mm + '/' + dd + '/' + yyyy;
        return formatted;
    };
    this.GetQueryStringParameter = function (key) {
        key = key.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
        var regex = new RegExp("[\\?&]" + key + "=([^&#]*)");
        var qs = regex.exec(window.location.href);
        if (qs == null) {
            return '';
        }
        else {
            return qs[1];
        }
    };
    this.UploadToDocumentFile = function (uploadFile) {
        if (uploadFile != null) {
            var fileName = uploadFile.name.split('\'');
            var fileNameParts = fileName[fileName.length - 1].split('.');
            var fileNamePartsExtIndex = fileNameParts.length - 1;
            var documentFile = {
                FileExtension: '.' + fileNameParts[fileNameParts.length - 1].toLowerCase(),
                OriginalName: fileNameParts[0].toString()
            };
            return documentFile;
        }
        else {
            this.AlertUser('Upload File is null.', 'No upload file was specified.', function () { return null; });
        }
    };
    this.Pad = function (number, length) {

        var str = '' + number;
        while (str.length < length) {
            str = '0' + str;
        }

        return str;

    };
    this.GetParentOfType = function (d, type) {
        var client = this,
            currentDoc = d;
        while (true) {
            this.GetDocument(currentDoc.ParentId, function (doc) {
                currentDoc = doc;
            }, function (err) {
                currentDoc = null;
            });
            if (client.DocumentTypesByName()[currentDoc.DocumentTypeId] == type || currentDoc == null) {
                break;
            }
        }
        return currentDoc;
    };
    this.AddMenuItem = function (itemName, icon, click, menu) {
        var navigation, menuItem, menuLink, menuIcon;
        if (menu == 'alt') {
            navigation = document.getElementById('navigation-alt');
        }
        else {
            navigation = document.getElementById('navigation-menu');
        }
        menuItem = document.createElement('li');
        navigation.appendChild(menuItem);
        menuLink = document.createElement('a');
        menuItem.appendChild(menuLink);
        menuIcon = document.createElement('i');
        menuLink.appendChild(menuIcon);
        menuIcon.className = icon;
        menuLink.onclick = click;
        if (itemName != 'notifications' && itemName != 'signout') {
            menuLink.appendChild(document.createTextNode(' ' + itemName));
        }
    };
    this.BuildMenu = function (user) {
        var client = this;
        var alt = document.getElementById('navigation-alt');
        var menu = document.getElementById('navigation-menu');
        alt.innerHTML = '';
        menu.innerHTML = '';
        if (user.Role == 'Guest') {
            client.AddMenuItem(user.FirstName + ' ' + user.LastName, 'fa fa-user', function () { return client.OpenDialog('profiledialog'); }, 'alt');
            client.AddMenuItem('signout', 'fa fa-sign-out', function () { client.SignOut(); }, 'alt');
        }
        else if (user.Role == 'Partner') {
            client.AddMenuItem(user.FirstName + ' ' + user.LastName, 'fa fa-user', function () { return openDialog('profiledialog'); }, 'alt');
            client.AddMenuItem('signout', 'fa fa-sign-out', function () { client.SignOut(); }, 'alt');
        }
        else if (user.Role == 'Standard User') {
            client.AddMenuItem('dashboard', 'fa fa-dashboard', function () { return client.OpenForm('dashboard', 'main_content'); }, 'menu');
            client.AddMenuItem('add document', 'fa fa-plus-circle', function () { return client.OpenDialog('adddocumentdialog'); }, 'menu');
            client.AddMenuItem('search', 'fa fa-search', function () { return client.OpenDialog('searchdialog'); }, 'menu');
            client.AddMenuItem('apps', 'fa fa-puzzle-piece', function () { return client.OpenForm('apps', 'main_content'); }, 'menu');
            client.AddMenuItem(user.FirstName + ' ' + user.LastName, 'fa fa-user', function () { return client.OpenDialog('profiledialog'); }, 'alt');
            client.AddMenuItem('notifications', 'fa fa-flag', function () { return client.OpenForm('notifications', 'main_content'); }, 'alt');
            client.AddMenuItem('signout', 'fa fa-sign-out', function () { client.SignOut(); }, 'alt');
        }
        else if (user.Role == 'Administrator') {
            client.AddMenuItem('dashboard', 'fa fa-dashboard', function () { return client.OpenForm('dashboard', 'main_content'); }, 'menu');
            client.AddMenuItem('add document', 'fa fa-plus-circle', function () { return client.OpenDialog('adddocumentdialog'); }, 'menu');
            client.AddMenuItem('search', 'fa fa-search', function () { return client.OpenDialog('searchdialog'); }, 'menu');
            client.AddMenuItem('apps', 'fa fa-puzzle-piece', function () { return client.OpenForm('apps', 'main_content'); }, 'menu');
            client.AddMenuItem('configuration', 'fa fa-gear', function () { return client.OpenForm('solutionconfigform', 'main_content'); }, 'menu');
            client.AddMenuItem(user.FirstName + ' ' + user.LastName, 'fa fa-user', function () { return client.OpenDialog('profiledialog'); }, 'alt');
            client.AddMenuItem('notifications', 'fa fa-flag', function () { return client.OpenForm('notifications', 'main_content'); }, 'alt');
            client.AddMenuItem('signout', 'fa fa-sign-out', function () { client.SignOut(); }, 'alt');
        }
        else if (user.Role == 'Service Manager') {
            client.AddMenuItem('dashboard', 'fa fa-dashboard', function () { return client.OpenForm('dashboard', 'main_content'); }, 'menu');
            client.AddMenuItem('add document', 'fa fa-plus-circle', function () { return client.OpenDialog('adddocumentdialog'); }, 'menu');
            client.AddMenuItem('search', 'fa fa-search', function () { return client.OpenDialog('searchdialog'); }, 'menu');
            client.AddMenuItem('apps', 'fa fa-puzzle-piece', function () { return client.OpenForm('apps', 'main_content'); }, 'menu');
            client.AddMenuItem('configuration', 'fa fa-gear', function () { return client.OpenForm('solutionconfigform', 'main_content'); }, 'menu');
            client.AddMenuItem('solutions', 'fa fa-globe', function () { return client.OpenForm('solutions', 'main_content'); }, 'menu');
            client.AddMenuItem(user.FirstName + ' ' + user.LastName, 'fa fa-user', function () { return client.OpenDialog('profiledialog'); }, 'alt');
            client.AddMenuItem('notifications', 'fa fa-flag', function () { return client.OpenForm('notifications', 'main_content'); }, 'alt');
            client.AddMenuItem('signout', 'fa fa-sign-out', function () { client.SignOut(); }, 'alt');
        }
        else {
            // load nothing
        }
    };
    this.DocumentTypesByName = function () {
        var documentTypesByName = { };
        for (var t in this.App.Schema.DocumentTypes) {
            var documentTypeByName = {
                DocumentTypeId: this.App.Schema.DocumentTypes[t].DocumentTypeId,
                IsChildType: this.App.Schema.DocumentTypes[t].IsChildType,
                Name: this.App.Schema.DocumentTypes[t].Name,
                Description: this.App.Schema.DocumentTypes[t].Description,
                DocumentProperties: {}
            };
            for (var p in this.App.Schema.DocumentTypes[t].DocumentProperties) {
                var property = {
                    DocumentPropertyId: this.App.Schema.DocumentTypes[t].DocumentProperties[p].DocumentPropertyId,
                    Name: this.App.Schema.DocumentTypes[t].DocumentProperties[p].Name,
                    DataType: this.App.Schema.DocumentTypes[t].DocumentProperties[p].DataType,
                    Description: this.App.Schema.DocumentTypes[t].DocumentProperties[p].Description,
                    FormatString: this.App.Schema.DocumentTypes[t].DocumentProperties[p].FormatString,
                    RegEx: this.App.Schema.DocumentTypes[t].DocumentProperties[p].RegEx,
                    RegExMessage: this.App.Schema.DocumentTypes[t].DocumentProperties[p].RegExMessage,
                    ListId: this.App.Schema.DocumentTypes[t].DocumentProperties[p].ListId
                };
                documentTypeByName.DocumentProperties[property.Name] = property;
            }
            documentTypesByName[documentTypeByName.Name] = documentTypeByName;
        }
        return documentTypesByName;
    };
    this.BuildSelect = function (options) {
        var select = document.createElement('select');
        select.className = options.ClassName;
        select.id = options.Id;
        for (var i = 0, l = options.Options.length; i < l; i++) {
            var opt = document.createElement('option');
            opt.value = options.Options[i].Value;
            opt.appendChild(document.createTextNode(options.Options[i].Name));
            select.appendChild(opt);
        }
        options.Target.appendChild(select);
    };
    this.ApplyTheme = function (bootstrapCssUrl) {
        var siteTheme = document.getElementById('site-theme');
        siteTheme.setAttribute('href', bootstrapCssUrl);
    };
    this.GenNewId = function (cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'utilities/id',
            type: 'GET',
            contentType: 'application/json;charset=utf-8'
        })
        .done(function (id) { cb(id); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.CropPage = function (did, fid, x, y, w, h, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'utilities/' + did + '/' + fid + '/' + x + '/' + y + '/' + w + '/' + h,
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function () { cb(); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.RemovePhotoTags = function (did, fid, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'utilities/' + did + '/' + fid,
            type: 'GET',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function () { cb(); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
    this.EDMDateToString = function (edmDate, cb, fb) {
        return edmDate.substring(5,7) + '/' + edmDate.substring(8,10) + '/' + edmDate.substring(0,4);
    };
    this.StringToEDMDate = function (stringDate, cb, fb) {
        var parts = stringDate.split('/');
        return this.Pad(parts[2], 4) + '-' + this.Pad(parts[0], 2) + '-' + this.Pad(parts[1], 2) + 'T00:00:00';
    };
    // controls
    this.PropertyGrid = function (target, options) {
        //var tgt = document.getElementById(target);
        //var table = document.createElement('table');
        var table = document.getElementById(target);
        table.innerHTML = "";
        table.className = 'table table-hover table-striped';
        //tgt.innerHTML = "";
        //tgt.appendChild(table);
        var header = table.createTHead();
        var headerRow = header.insertRow();
        var body = document.createElement('tbody');
        table.appendChild(body);
        for (var c = 0, cc = options.columns.length; c < cc; c++) {
            var cell = document.createElement('td');
            headerRow.appendChild(cell);
            cell.appendChild(document.createTextNode(options.columns[c].Name));
        }
        this.Load = function (data) {
            for (var o in data) {
                var row = body.insertRow(-1);
                for (var l = 0, u = options.columns.length; l < u; l++) {
                    var cell = document.createElement('td');
                    cell.appendChild(document.createTextNode(data[o][options.columns[l].Id]));
                    row.appendChild(cell);
                }
                row.onclick = (function (a) { return function () { options.onSelect(a); } })(data[o]);
            }
        }
    };
    this.Uploader = function () {

        var _file, _blockSize, _writeUrl, _progress, _uploaded, _blockCount, _fileSize, _onOk;

        this.Upload = function (file, blockSize, writeUrl, progress, onOk) {
            _file = file;
            _blockSize = blockSize;
            _writeUrl = writeUrl;
            _progress = progress;
            _onOk = onOk;
            _uploaded = 0;
            _fileSize = _file.size;
            _progress.style.width = '0%';
            _blockCount = Math.ceil(_fileSize / _blockSize);
            uploadFile();
        }

        function uploadFile() {
            if (_blockCount > 1) {
                var chunck;
                for (var b = 0; b < _blockCount; b++) {
                    var start = b * _blockSize;
                    var stop = start + _blockSize;
                    if (stop > _fileSize) {
                        stop = _fileSize;
                    }
                    if (_file.slice) {
                        chunck = _file.slice(start, stop);
                    }
                    else if (_file.webkitSlice) {
                        chunck = _file.webkitSlice(start, stop);
                    }
                    else if (_file.mozSlice) {
                        chunck = _file.mozSlice(start, stop);
                    }
                    readFilePart(b, chunck);
                }
            }
            else {
                readFile(_file);
            }
        }

        function readFilePart(blockId, blob) {
            var reader = new FileReader();
            reader.onloadend = function (e) {
                if (e.target.readyState == FileReader.DONE) {
                    var buffer = e.target.result;
                    var bytes = new Uint8Array(buffer);
                    uploadFileBlockAsBinary(blockId, bytes);
                }
            };
            reader.readAsArrayBuffer(blob);
        }

        function readFile(blob) {
            var reader = new FileReader();
            reader.onloadend = function (e) {
                if (e.target.readyState == FileReader.DONE) {
                    var buffer = e.target.result;
                    var bytes = new Uint8Array(buffer);
                    uploadFileAsBinary(bytes);
                }
            };
            reader.readAsArrayBuffer(blob);
        }

        function uploadFileAsBinary(fileData) {
            $.ajax({
                url: _writeUrl,
                type: 'PUT',
                data: fileData,
                processData: false,
                headers: {
                    'x-ms-version': '2014-02-14',
                    'x-ms-blob-type': 'BlockBlob'
                }
            })
            .done(function (a) {
                _progress.style.width = '100%';
                _onOk(a);
            })
            .fail(function (response, msg) {
                return handleError(response, msg);
            })
            .always(function () { });
        }

        function uploadFileBlockAsBinary(blockId, fileData) {
            $.ajax({
                url: _writeUrl + '&comp=block&blockid=' + pad(blockId, 8),
                type: 'PUT',
                data: fileData,
                processData: false,
                headers: {
                    'x-ms-version': '2014-02-14'
                }
            })
            .done(function () {
                _uploaded++;
                _progress.style.width = (_uploaded / _blockCount) * 100 + '%';
                if (_uploaded == _blockCount)
                { commitBlocks(); }
            })
            .fail(function (response, msg) {
                return handleError(response, msg);
            })
            .always(function () { });
        }

        function commitBlocks() {
            var body = '<?xml version="1.0" encoding="utf-8"?><BlockList>';
            for (var i = 0; i < _blockCount; i++) {
                body += '<Latest>' + pad(i, 8) + '</Latest>';
            }
            body += '</BlockList>';

            $.ajax({
                url: _writeUrl + '&comp=blocklist',
                type: 'PUT',
                data: body,
                headers: {
                    'x-ms-version': '2014-02-14'
                }
            })
            .done(function (a) {
                _progress.style.width = '100%';
                _onOk(a);
            })
            .fail(function (response, msg) {
                return handleError(response, msg);
            })
            .always(function () { });
        }

        function pad(number, length) {

            var str = '' + number;
            while (str.length < length) {
                str = '0' + str;
            }

            return str;

        }
    };
    this.FileTable = function (target) {
        var tar = document.getElementById(target);
        var table = document.createElement('table');
        table.className = 'table table-hover table-striped';
        tar.appendChild(table);
        var header = table.createTHead();
        var headerRow = header.insertRow();
        var body = document.createElement('tbody');
        table.appendChild(body);
        var downloadHeaderCell = document.createElement('td');
        headerRow.appendChild(downloadHeaderCell);
        downloadHeaderCell.setAttribute('width', '50px');
        var reviseHeaderCell = document.createElement('td');
        headerRow.appendChild(reviseHeaderCell);
        reviseHeaderCell.setAttribute('width', '50px');
        var nameHeaderCell = document.createElement('td');
        headerRow.appendChild(nameHeaderCell);
        nameHeaderCell.appendChild(document.createTextNode('Name'));
        var sizeHeaderCell = document.createElement('td');
        headerRow.appendChild(sizeHeaderCell);
        sizeHeaderCell.appendChild(document.createTextNode('Size (bytes)'));
        this.Load = function (doc) {
            for (var i = 0, c = doc.Files.length; i < c; i++) {
                var file = doc.Files[i];
                var row = body.insertRow(-1);
                var downloadCell = document.createElement('td');
                row.appendChild(downloadCell);
                var downloadButton = document.createElement('button');
                downloadCell.appendChild(downloadButton);
                downloadButton.className = 'btn btn-primary btn-xs';
                var downloadIcon = document.createElement('i');
                downloadButton.appendChild(downloadIcon);
                downloadIcon.className = 'fa fa-cloud-download';
                var reviseCell = document.createElement('td');
                row.appendChild(reviseCell);
                var reviseButton = document.createElement('button');
                reviseCell.appendChild(reviseButton);
                reviseButton.className = 'btn btn-success btn-xs';
                var reviseIcon = document.createElement('i');
                reviseButton.appendChild(reviseIcon);
                reviseIcon.className = 'fa fa-cloud-upload';
                var nameCell = document.createElement('td');
                row.appendChild(nameCell);
                if (file.OriginalName != null) {
                    nameCell.appendChild(document.createTextNode(file.OriginalName));
                }
                else {
                    nameCell.appendChild(document.createTextNode(''));
                }
                var sizeCell = document.createElement('td');
                row.appendChild(sizeCell);
                sizeCell.appendChild(document.createTextNode(file.Size));
            }
        }
    };
    this.SearchGrid = function (target, resultsLabel) {
        var doit;
        var tgt = document.getElementById(target);
        var filters = document.createElement('div');
        filters.className = 'col-md-3 filter-container';
        tgt.appendChild(filters);
        var fixedFilters = document.createElement('div');
        fixedFilters.className = 'fixed-filters';
        filters.appendChild(fixedFilters);
        var searchLabel = document.createElement('h3');
        searchLabel.appendChild(document.createTextNode(resultsLabel));
        fixedFilters.appendChild(searchLabel);
        var filterHeader = document.createElement('h4');
        filterHeader.appendChild(document.createTextNode('Filters'));
        fixedFilters.appendChild(filterHeader);
        // document type filter
        for (var t in ods.App.SearchResults.CollectionCounts.DocumentTypeId) {
            var filter = document.createElement('span');    //span
            filter.className = 'col-md-12 s-filter';
            filter.onclick = (function (a) { return function () { filterSearch(a); } })(ods.App.Schema.DocumentTypes[t].DocumentTypeId);

            filter.appendChild(document.createTextNode(ods.App.Schema.DocumentTypes[t].Name + ' (' + ods.App.SearchResults.CollectionCounts.DocumentTypeId[t] + ')'));
            fixedFilters.appendChild(filter);
        }

        if (ods.App.Query.Filter != null) {
            var clearFiltersButton = document.createElement('button');
            clearFiltersButton.className = 'btn btn-default btn-xs';
            clearFiltersButton.appendChild(document.createTextNode('Remove Filter'));
            clearFiltersButton.onclick = (function () { return function () { removeFilter(); } })();
            fixedFilters.appendChild(clearFiltersButton);
        }

        var docs = document.createElement('div');
        docs.className = 'col-md-9';
        tgt.appendChild(docs);

        window.onscroll = function () {
            clearTimeout(doit);
            doit = setTimeout(function () {
                if (bottomOfPageChecker()) {
                    if (!ods.App.SearchResultsLoaded) {
                        loadMore(ods.App.Query.CurrentPage + 1);
                    }
                }
            }, 100);
        };

        this.Load = function load() {
            renderResults(0, ods.App.SearchResults);
        }

        function removeFilter() {
            ods.App.Query.Filter = null;
            refreshSearch();
        }

        function renderResults(page, results) {
            for (var i = 0, l = results.Documents.length; i < l; i++) {
                var doc = document.createElement('div');
                doc.className = 'col-md-12 s-result-container';
                //doc.onclick = (function (a) {
                //    return function () {
                //        ods.App.Document = a;
                //        ods.OpenForm('documentdetailsform', 'main_content');
                //    }
                //})(results.Documents[i]);
                doc.onclick = (function (a) {
                    return function () {
                        ods.GetDocument(a.DocumentId, function (selectedDoc) {
                            ods.App.Document = selectedDoc;
                            // check to see if a custom form exists for the document type
                            // prior to loading the defaults below
                            //if (ods.App.Document.Files.length > 0) {
                            //    ods.OpenForm('documentdetailsform', 'main_content');
                            //}
                            //else {
                            //    alert('doc has no files');
                            //}
                            ods.OpenForm('documentdetailsform', 'main_content');
                        });
                    }
                })(results.Documents[i]);
                docs.appendChild(doc);
                var header = document.createElement('div');
                header.className = 'col-md-12';
                doc.appendChild(header);
                var headerTitle = document.createElement('span');    //span
                headerTitle.className = 's-result-header';
                header.appendChild(headerTitle);
                headerTitle.appendChild(document.createTextNode(ods.App.Schema.DocumentTypes[results.Documents[i].DocumentTypeId].Name));

                var createdOnProperty = document.createElement('div');
                createdOnProperty.className = 'col-md-12 s-result';
                doc.appendChild(createdOnProperty);
                var createdOnLabel = document.createElement('label');
                createdOnLabel.className = 's-result-label';
                createdOnProperty.appendChild(createdOnLabel);
                createdOnLabel.appendChild(document.createTextNode('Created On:'));
                var createdOnPropertyValue = document.createElement('span');    //span
                createdOnPropertyValue.className = 's-result-system';
                createdOnProperty.appendChild(createdOnPropertyValue);
                var createdOnValue = ods.FormatDate(ods.GetDate(results.Documents[i].CreatedOn));
                createdOnPropertyValue.appendChild(document.createTextNode(createdOnValue));
                createdOnPropertyValue.setAttribute('title', createdOnValue);

                var nameProperty = document.createElement('div');
                nameProperty.className = 'col-md-12 s-result';
                doc.appendChild(nameProperty);
                var nameLabel = document.createElement('label');
                nameLabel.className = 's-result-label';
                nameProperty.appendChild(nameLabel);
                nameLabel.appendChild(document.createTextNode('Name:'));
                var namePropertyValue = document.createElement('span');    //span
                namePropertyValue.className = 's-result-system';
                nameProperty.appendChild(namePropertyValue);
                var nameValue = results.Documents[i].Name;
                namePropertyValue.appendChild(document.createTextNode(nameValue));
                namePropertyValue.setAttribute('title', nameValue);

                // custom labels
                for (var f in ods.App.Schema.DocumentTypes[results.Documents[i].DocumentTypeId].DocumentProperties) {
                    // move code from below to up here  
                    var field = ods.App.Schema.DocumentTypes[results.Documents[i].DocumentTypeId].DocumentProperties[f];
                    var fieldValue = '';
                    if (results.Documents[i].Fields[field.DocumentPropertyId] != undefined) {
                        if (field.DataType == 'text') {
                            fieldValue = results.Documents[i].Fields[field.DocumentPropertyId];
                            //for (var v = 0, c = results.Documents[i].Fields[field.DocumentPropertyId].length; v < c; v++) {
                            //    fieldValue += results.Documents[i].Fields[field.DocumentPropertyId][v] + ' ';
                            //}
                        }
                        else if (field.DataType == 'datetime') {
                            fieldValue = results.Documents[i].Fields[field.DocumentPropertyId];
                            //for (var v = 0, c = results.Documents[i].Fields[field.DocumentPropertyId].length; v < c; v++) {
                            //    fieldValue += ods.FormatDate(ods.GetDate(results.Documents[i].Fields[field.DocumentPropertyId][v])) + ' ';
                            //}
                        }
                        else {
                            fieldValue = results.Documents[i].Fields[field.DocumentPropertyId];
                            //for (var v = 0, c = results.Documents[i].Fields[field.DocumentPropertyId].length; v < c; v++) {
                            //    fieldValue += results.Documents[i].Fields[field.DocumentPropertyId][v] + ' ';
                            //}
                        }
                    }
                    else {
                        fieldValue = '';
                    }

                    var property = document.createElement('div');
                    property.className = 'col-md-12 s-result';
                    doc.appendChild(property);
                    var label = document.createElement('label');
                    label.className = 's-result-label';
                    property.appendChild(label);
                    label.appendChild(document.createTextNode(field.Name + ':'));
                    var propertyValue = document.createElement('span');    //span
                    propertyValue.className = 's-result-value';
                    property.appendChild(propertyValue);
                    propertyValue.appendChild(document.createTextNode(fieldValue));
                    propertyValue.setAttribute('title', fieldValue);
                }
            }
            if (results.Documents.length == 0) {
                ods.App.SearchResultsLoaded = true;
            }
        }

        function loadMore(page) {
            ods.App.Query.CurrentPage = page;
            ods.Search(ods.App.Query, function (results) {
                ods.App.SearchResults.Documents.push.apply(ods.App.SearchResults.Documents, results.Documents);
                renderResults(ods.App.Query.CurrentPage, results);
            });
        }

        function filterSearch(filter) {
            ods.App.Query.Filter = 'DocumentTypeId eq \'' + filter + '\'';
            refreshSearch();
        }

        function refreshSearch() {
            ods.App.SearchResultsLoaded = false;
            ods.App.Query.CurrentPage = 0;
            ods.Search(ods.App.Query, function (results) {
                ods.App.SearchResults = results;
                ods.OpenForm('resultsform', 'main_content');
            });
        }

        // problem with this method in safari and possibly chrome
        // http://stackoverflow.com/questions/1830080/jquery-scrolltop-doesnt-seem-to-work-in-safari-or-chrome-windows
        function bottomOfPageChecker() {
            var totalHeight, currentScroll, visibleHeight;

            if (document.documentElement.scrollTop)
            { currentScroll = document.documentElement.scrollTop; }
            else
            { currentScroll = document.body.scrollTop; }

            totalHeight = document.body.offsetHeight;
            visibleHeight = document.documentElement.clientHeight;

            if (totalHeight <= currentScroll + visibleHeight) {
                return true;
            }
            else {
                return false;
            }
        }
    };
    this.SearchGrid2 = function (target, resultsLabel) {
        var docType, doit, finalPageLoaded = false;
        var cf = '';
        var tgt = document.getElementById(target);
        tgt.innerHTML = '';
        var filters = document.createElement('div');
        filters.className = 'doc-type-list';
        tgt.appendChild(filters);
        if (ods.App.Query.SortExpression == undefined) {
            ods.App.Query.SortExpression = 'CreatedOn';
            ods.App.Query.SortDirection = 'desc';
        }

        if (ods.App.Filters != null) {
            // document type filter
            for (var f in ods.App.Filters) {
                if (ods.App.Filters[f].Selected) {
                    docType = ods.App.Filters[f].DocumentType;
                }
                var filter = document.createElement('button');
                filter.type = 'button';
                if (ods.App.Filters[f].Selected) {
                    filter.className = 'btn btn-success btn-lg full-button-2';
                }
                else {
                    filter.className = 'btn btn-default btn-sm full-button-2';
                }
                filter.onclick = (function (a) { return function () { filterSearch(a); } })(f);
                filter.appendChild(document.createTextNode(ods.App.Filters[f].DocumentType.Name + ' (' + ods.App.Filters[f].Count + ')'));
                filters.appendChild(filter);
            }
        }
        else {
            docType = ods.App.SearchedType;
        }

        var docsTable = document.createElement('table');
        docsTable.className = 'super-table table-hover table-striped full-table';
        tgt.appendChild(docsTable);
        var header = docsTable.createTHead();
        var headerRow = header.insertRow();

        var cHeader = document.createElement('th');
        cHeader.className = 'count-labels';

        var lfic = document.createElement('span');
        lfic.title = 'File Count';
        lfic.className = 'label label-primary';
        var fic = document.createElement('i');
        fic.className = 'fa fa-file-o';
        lfic.appendChild(fic);
        cHeader.appendChild(lfic);

        var lcic = document.createElement('span');
        lcic.title = 'Comment Count';
        lcic.className = 'label label-success';
        var cic = document.createElement('i');
        cic.className = 'fa fa-comment-o';
        lcic.appendChild(cic);
        cHeader.appendChild(lcic);

        var lvic = document.createElement('span');
        lvic.title = 'Version Count';
        lvic.className = 'label label-info';
        var vic = document.createElement('i');
        vic.className = 'fa fa-files-o';
        lvic.appendChild(vic);
        cHeader.appendChild(lvic);

        headerRow.appendChild(cHeader);

        for (var p in docType.DocumentProperties) {
            var property = docType.DocumentProperties[p];
            var ph = document.createElement('th');
            var pa = document.createElement('a');
            pa.appendChild(document.createTextNode(property.Name));
            pa.onclick = (function (a) {
                return function () {
                    if (ods.App.Query.SortExpression == a) {
                        if (ods.App.Query.SortDirection == 'desc') {
                            ods.App.Query.SortDirection = 'asc';
                        }
                        else {
                            ods.App.Query.SortDirection = 'desc';
                        }
                    }
                    else {
                        ods.App.Query.SortDirection = 'asc';
                    }
                    ods.App.Query.SortExpression = a;
                    refreshSearch();
                }
            })(property.DocumentPropertyId);
            ph.appendChild(pa);
            headerRow.appendChild(ph);
        }

        var coh = document.createElement('th');
        var coha = document.createElement('a');
        coha.appendChild(document.createTextNode('Created On'));
        coha.onclick = (function (a) {
            return function () {
                if (ods.App.Query.SortExpression == a) {
                    if (ods.App.Query.SortDirection == 'desc') {
                        ods.App.Query.SortDirection = 'asc';
                    }
                    else {
                        ods.App.Query.SortDirection = 'desc';
                    }
                }
                else {
                    ods.App.Query.SortDirection = 'asc';
                }
                ods.App.Query.SortExpression = a;
                refreshSearch();
            }
        })('CreatedOn');
        coh.appendChild(coha);
        headerRow.appendChild(coh);

        var docs = document.createElement('tbody');
        docsTable.appendChild(docs);

        if (ods.App.ResultsFilteredBy != null) {
            tgt.appendChild(docsContainer);
        }

        docs.onscroll = function () {
            if (!finalPageLoaded) {
                clearTimeout(doit);
                doit = setTimeout(function () {
                    if (docs.scrollTop + docs.clientHeight == docs.scrollHeight) {
                        loadMore(ods.App.Query.CurrentPage + 1);
                    }
                }, 100);
            }
        };
         
        this.Load = function load() {
            renderResults(0, ods.App.SearchResults);
        }

        //function applySort_old(docType) {
        //    // apply default sort if specified
        //    if (docType.DefaultSortExpression != null && docType.DefaultSortDirection != null) {
        //        ods.App.Query.SortExpression = docType.DefaultSortExpression;
        //        ods.App.Query.SortDirection = docType.DefaultSortDirection;
        //    }
        //    else {
        //        ods.App.Query.SortExpression = 'CreatedOn';
        //        ods.App.Query.SortDirection = 'desc';
        //    }
        //}

        //function applySort() {
        //    // apply default sort if specified
        //    var sortExpression = document.getElementById('ddl-sort-expression').value;
        //    var sortDirection = document.getElementById('ddl-sort-direction').value;
        //    if (sortExpression != null && sortDirection != null) {
        //        ods.App.Query.SortExpression = sortExpression;
        //        ods.App.Query.SortDirection = sortDirection;
        //    }
        //    else {
        //        ods.App.Query.SortExpression = 'CreatedOn';
        //        ods.App.Query.SortDirection = 'desc';
        //    }
        //}

        function loadMore(page) {
            ods.App.Query.CurrentPage = page;
            ods.Search(ods.App.Query, function (results) {
                ods.App.SearchResults.Documents.push.apply(ods.App.SearchResults.Documents, results.Documents);
                if (results.Documents.length == 0) {
                    finalPageLoaded = true;
                    ods.NotifyUser('End of Results', 'You have reached the end of the results list.');
                }
                renderResults(ods.App.Query.CurrentPage, results);
            });
        }

        function renderResults(page, results) {
            if (results.Documents.length > 0) {
                for (var i = 0, l = results.Documents.length; i < l; i++) {

                    var doc = docs.insertRow(-1);

                    var countsCell = document.createElement('td');
                    countsCell.className = 'count-labels';
                    var countFile = document.createElement('span');
                    countFile.className = 'label label-primary';
                    countFile.appendChild(document.createTextNode(results.Documents[i].FileCount));
                    countsCell.appendChild(countFile);

                    var countFile = document.createElement('span');
                    countFile.className = 'label label-success';
                    countFile.appendChild(document.createTextNode(results.Documents[i].CommentCount));
                    countsCell.appendChild(countFile);

                    var countFile = document.createElement('span');
                    countFile.className = 'label label-info';
                    countFile.appendChild(document.createTextNode(results.Documents[i].VersionCount));
                    countsCell.appendChild(countFile);

                    doc.appendChild(countsCell);

                    for (var f in docType.DocumentProperties) {
                        var field = docType.DocumentProperties[f];
                        var fieldValue = '';
                        if (results.Documents[i].Fields[field.DocumentPropertyId] != undefined) {
                            if (field.DataType == 'text') {
                                fieldValue = results.Documents[i].Fields[field.DocumentPropertyId];
                            }
                            else if (field.DataType == 'datetime') {
                                fieldValue = ods.EDMDateToString(results.Documents[i].Fields[field.DocumentPropertyId]);
                            }
                            else {
                                fieldValue = results.Documents[i].Fields[field.DocumentPropertyId];
                            }
                        }
                        else {
                            fieldValue = '';
                        }
                        var cellFieldValue = document.createElement('td');
                        cellFieldValue.appendChild(document.createTextNode(fieldValue));
                        doc.appendChild(cellFieldValue);
                    }

                    var cellCreatedOn = document.createElement('td');
                    cellCreatedOn.appendChild(document.createTextNode(ods.FormatDate(ods.GetDate(results.Documents[i].CreatedOn))));
                    doc.appendChild(cellCreatedOn);

                    doc.onclick = (function (a) {
                        return function () {
                            ods.GetDocument(a.DocumentId, function (selectedDoc) {
                                ods.App.Document = selectedDoc;
                                ods.Temp.Classifying = false;
                                // check to see if a custom form exists for the document type
                                // prior to loading the defaults below
                                //if (ods.App.Document.Files.length > 0) {
                                //    ods.OpenForm('documentdetailsform', 'main_content');
                                //}
                                //else {
                                //    alert('doc has no files');
                                //}
                                ods.OpenForm('documentdetailsform', 'main_content');
                            });
                        }
                    })(results.Documents[i]);
                }
            }
        }

        function filterSearch(filter) {
            for (var f in ods.App.Filters) {
                if (f == filter) {
                    ods.App.Filters[f].Selected = true;
                }
                else {
                    ods.App.Filters[f].Selected = false;
                }
            }
            ods.App.Query.Filter = 'DocumentStatus ne \'deleted\' and DocumentTypeId eq \'' + filter + '\'';
            refreshSearch();
        }

        function refreshSearch() {
            ods.App.Query.CurrentPage = 0;
            ods.Search(ods.App.Query, function (results) {
                ods.App.SearchResults = results;
                ods.OpenForm('resultsform', 'main_content');
            });
        }
    };
    // this works, just not happy with formatting
    this.SearchGrid3 = function (target, resultsLabel) {
        var doit;
        var tgt = document.getElementById(target);
        var filters = document.createElement('div');
        filters.className = 'col-md-3 filter-container';
        tgt.appendChild(filters);
        var fixedFilters = document.createElement('div');
        fixedFilters.className = 'fixed-filters';
        filters.appendChild(fixedFilters);
        var searchLabel = document.createElement('h3');
        searchLabel.appendChild(document.createTextNode(resultsLabel));
        fixedFilters.appendChild(searchLabel);
        var filterHeader = document.createElement('h4');
        filterHeader.appendChild(document.createTextNode('Document Types'));
        fixedFilters.appendChild(filterHeader);
        // document type filter
        for (var t in ods.App.SearchResults.CollectionCounts.DocumentTypeId) {
            var filter = document.createElement('span');    //span
            filter.className = 'col-md-12 s-filter';
            filter.onclick = (function (a) { return function () { filterSearch(a); } })(ods.App.Schema.DocumentTypes[t].DocumentTypeId);

            filter.appendChild(document.createTextNode(ods.App.Schema.DocumentTypes[t].Name + ' (' + ods.App.SearchResults.CollectionCounts.DocumentTypeId[t] + ')'));
            fixedFilters.appendChild(filter);
        }

        if (ods.App.Query.Filter != null) {
            var clearFiltersButton = document.createElement('button');
            clearFiltersButton.className = 'btn btn-default btn-xs';
            clearFiltersButton.appendChild(document.createTextNode('Remove Filter'));
            clearFiltersButton.onclick = (function () { return function () { removeFilter(); } })();
            fixedFilters.appendChild(clearFiltersButton);
        }

        var docs = document.createElement('div');
        docs.className = 'col-md-9';
        tgt.appendChild(docs);

        window.onscroll = function () {
            clearTimeout(doit);
            doit = setTimeout(function () {
                if (bottomOfPageChecker()) {
                    if (!ods.App.SearchResultsLoaded) {
                        loadMore(ods.App.Query.CurrentPage + 1);
                    }
                }
            }, 100);
        };

        this.Load = function load() {
            renderResults(0, ods.App.SearchResults);
        }

        function removeFilter() {
            ods.App.Query.Filter = null;
            refreshSearch();
        }

        function renderResults(page, results) {
            for (var i = 0, l = results.Documents.length; i < l; i++) {
                var doc = document.createElement('div');
                doc.className = 'col-md-12 s-result-2-doc';
                doc.onclick = (function (a) {
                    return function () {
                        ods.GetDocument(a.DocumentId, function (selectedDoc) {
                            ods.App.Document = selectedDoc;
                            // check to see if a custom form exists for the document type
                            // prior to loading the defaults below
                            //if (ods.App.Document.Files.length > 0) {
                            //    ods.OpenForm('documentdetailsform', 'main_content');
                            //}
                            //else {
                            //    alert('doc has no files');
                            //}
                            ods.OpenForm('documentdetailsform', 'main_content');
                        });
                    }
                })(results.Documents[i]);
                docs.appendChild(doc);
                var header = document.createElement('div');
                header.className = 'col-md-12 s-result-entity';
                doc.appendChild(header);
                var headerTitle = document.createElement('span');    //span
                headerTitle.className = 's-result-2-lnk';
                header.appendChild(headerTitle);
                var nameValue = results.Documents[i].Name;
                headerTitle.appendChild(document.createTextNode(ods.App.Schema.DocumentTypes[results.Documents[i].DocumentTypeId].Name));  //


                var counts = document.createElement('div');
                counts.className = 'col-md-12 s-result-2-fld';
                doc.appendChild(counts);

                var fileCount = document.createElement('span');
                fileCount.className = 'label label-primary';
                if (results.Documents[i].FileCount == 1) {
                    fileCount.appendChild(document.createTextNode(results.Documents[i].FileCount + ' file'));
                }
                else {
                    fileCount.appendChild(document.createTextNode(results.Documents[i].FileCount + ' files'));
                }
                counts.appendChild(fileCount);

                var commentCount = document.createElement('span');
                commentCount.className = 'label label-success';
                if (results.Documents[i].CommentCount == 1) {
                    commentCount.appendChild(document.createTextNode(results.Documents[i].CommentCount + ' comment'));
                }
                else {
                    commentCount.appendChild(document.createTextNode(results.Documents[i].CommentCount + ' comments'));
                }
                counts.appendChild(commentCount);

                var versionCount = document.createElement('span');
                versionCount.className = 'label label-info';
                if (results.Documents[i].VersionCount == 1) {
                    versionCount.appendChild(document.createTextNode(results.Documents[i].VersionCount + ' version'));
                }
                else {
                    versionCount.appendChild(document.createTextNode(results.Documents[i].VersionCount + ' versions'));
                }
                counts.appendChild(versionCount);

                //var createdOnLabel = document.createElement('label');
                //createdOnLabel.className = 's-result-label';
                //createdOnProperty.appendChild(createdOnLabel);
                //createdOnLabel.appendChild(document.createTextNode('Created On:'));

                //var nameProperty = document.createElement('div');
                //nameProperty.className = 'col-md-12 s-result';
                //doc.appendChild(nameProperty);
                //var nameLabel = document.createElement('label');
                //nameLabel.className = 's-result-label';
                //nameProperty.appendChild(nameLabel);
                //nameLabel.appendChild(document.createTextNode('Name:'));
                //var namePropertyValue = document.createElement('span');    //span
                //namePropertyValue.className = 's-result-system';
                //nameProperty.appendChild(namePropertyValue);
                //var nameValue = results.Documents[i].Name;
                //namePropertyValue.appendChild(document.createTextNode(ods.App.Schema.DocumentTypes[results.Documents[i].DocumentTypeId].Name));
                //namePropertyValue.setAttribute('title', nameValue);

                //var property = document.createElement('div');
                //property.className = 'col-md-12 s-result-2-fld';
                //doc.appendChild(property);

                // custom labels
                for (var f in ods.App.Schema.DocumentTypes[results.Documents[i].DocumentTypeId].DocumentProperties) {
                    // move code from below to up here  
                    var field = ods.App.Schema.DocumentTypes[results.Documents[i].DocumentTypeId].DocumentProperties[f];
                    var fieldValue = '';
                    if (results.Documents[i].Fields[field.DocumentPropertyId] != undefined) {
                        if (field.DataType == 'text') {
                            fieldValue = results.Documents[i].Fields[field.DocumentPropertyId];
                            //for (var v = 0, c = results.Documents[i].Fields[field.DocumentPropertyId].length; v < c; v++) {
                            //    fieldValue += results.Documents[i].Fields[field.DocumentPropertyId][v] + ' ';
                            //}
                        }
                        else if (field.DataType == 'datetime') {
                            //fieldValue = results.Documents[i].Fields[field.DocumentPropertyId];

                            fieldValue = ods.EDMDateToString(results.Documents[i].Fields[field.DocumentPropertyId]);

                            //for (var v = 0, c = results.Documents[i].Fields[field.DocumentPropertyId].length; v < c; v++) {
                            //    fieldValue += ods.FormatDate(ods.GetDate(results.Documents[i].Fields[field.DocumentPropertyId][v])) + ' ';
                            //}
                        }
                        else {
                            fieldValue = results.Documents[i].Fields[field.DocumentPropertyId];
                            //for (var v = 0, c = results.Documents[i].Fields[field.DocumentPropertyId].length; v < c; v++) {
                            //    fieldValue += results.Documents[i].Fields[field.DocumentPropertyId][v] + ' ';
                            //}
                        }
                    }
                    else {
                        fieldValue = '';
                    }

                    //var fieldContainer = document.createElement('div');
                    //fieldContainer.className = 'col-md-12';
                    //var fieldGroup = document.createElement('div');
                    //fieldGroup.className = 'form-group';
                    //var fieldlbl = document.createElement('label');
                    //fieldlbl.appendChild(document.createTextNode(field.Name));
                    var fieldval = document.createElement('div');
                    fieldval.className = 'col-md-12 results-field-value';
                    var fieldlbl = document.createElement('span');
                    fieldlbl.className = 'results-sys-lbl';
                    fieldlbl.appendChild(document.createTextNode(field.Name + ' : '));
                    fieldval.appendChild(fieldlbl);
                    fieldval.appendChild(document.createTextNode(fieldValue));
                    //fieldGroup.appendChild(fieldlbl);
                    //fieldGroup.appendChild(fieldval);
                    //fieldContainer.appendChild(fieldGroup);
                    doc.appendChild(fieldval);


                    //var label = document.createElement('label');
                    //label.className = 's-result-label';
                    //property.appendChild(label);
                    //label.appendChild(document.createTextNode(field.Name + ':'));

                    //var propertyValue = document.createElement('span');    //span
                    //propertyValue.className = 'label label-default';
                    //propertyValue.appendChild(document.createTextNode(fieldValue));
                    //property.appendChild(propertyValue);
                    //propertyValue.setAttribute('title', fieldValue);
                }


                var createdOnValue = ods.FormatDate(ods.GetDate(results.Documents[i].CreatedOn));
                var createdOnProperty = document.createElement('div');
                createdOnProperty.className = 'col-md-12 results-sys-lbl';
                createdOnProperty.appendChild(document.createTextNode(createdOnValue + ' ' + nameValue));
                doc.appendChild(createdOnProperty);

                //var createdOnPropertyValue = document.createElement('span');    //span
                ////createdOnPropertyValue.className = 's-result-2-sys';
                //createdOnProperty.appendChild(createdOnPropertyValue);
                //createdOnPropertyValue.appendChild(document.createTextNode(createdOnValue + ' ' + nameValue));
                //createdOnPropertyValue.setAttribute('title', createdOnValue + ' ' + nameValue);
            }
            if (results.Documents.length == 0) {
                ods.App.SearchResultsLoaded = true;
            }
        }

        function loadMore(page) {
            ods.App.Query.CurrentPage = page;
            ods.Search(ods.App.Query, function (results) {
                ods.App.SearchResults.Documents.push.apply(ods.App.SearchResults.Documents, results.Documents);
                renderResults(ods.App.Query.CurrentPage, results);
            });
        }

        function filterSearch(filter) {
            ods.App.Query.Filter = 'DocumentTypeId eq \'' + filter + '\'';
            refreshSearch();
        }

        function refreshSearch() {
            ods.App.SearchResultsLoaded = false;
            ods.App.Query.CurrentPage = 0;
            ods.Search(ods.App.Query, function (results) {
                ods.App.SearchResults = results;
                ods.OpenForm('resultsform', 'main_content');
            });
        }

        // problem with this method in safari and possibly chrome
        // http://stackoverflow.com/questions/1830080/jquery-scrolltop-doesnt-seem-to-work-in-safari-or-chrome-windows
        function bottomOfPageChecker() {
            var totalHeight, currentScroll, visibleHeight;

            if (document.documentElement.scrollTop)
            { currentScroll = document.documentElement.scrollTop; }
            else
            { currentScroll = document.body.scrollTop; }

            totalHeight = document.body.offsetHeight;
            visibleHeight = document.documentElement.clientHeight;

            if (totalHeight <= currentScroll + visibleHeight) {
                return true;
            }
            else {
                return false;
            }
        }
    };
    this.OneGrid = function (target, options) {
        var tgt = document.getElementById(target);
        var table = document.createElement('table');
        table.className = 'table table-hover table-striped';
        tgt.innerHTML = "";
        tgt.appendChild(table);
        var cc = options.columns.length;
        var header = table.createTHead();
        var headerRow = header.insertRow();
        var body = document.createElement('tbody');

        table.appendChild(body);
        if (cc > 0) {
            for (var c = 0; c < cc; c++) {
                var cell = document.createElement('td');
                headerRow.appendChild(cell);
                cell.appendChild(document.createTextNode(options.columns[c].Name));
            }
        }

        // load data from datasource
        this.Load = function load(data) {
            if (data != null) {
                var ct = data.length;
                for (var r = 0; r < ct; r++) {
                    var row = body.insertRow(-1);
                    if (cc > 0) {
                        for (var c = 0; c < cc; c++) {
                            var cell, val;
                            cell = document.createElement('td');
                            if (options.columns[c].DataType == 'text') {
                                val = data[r][options.columns[c].Id];
                            }
                            else if (options.columns[c].DataType == 'date') {
                                val = new Date(data[r][options.columns[c].Id]).toLocaleDateString('en-US');
                                //if (val == 'Invalid Date') { val = getDate(data[r][options.columns[c].Id]).toLocaleDateString('en-US'); }
                            }
                            else if (options.columns[c].DataType == 'number') {
                                val = new Number(data[r][options.columns[c].Id]).toLocaleDateString('en-US');
                            }
                            else {
                                val = data[r][options.columns[c].Id];
                            }
                            row.appendChild(cell);
                            if (val != null) {
                                cell.appendChild(document.createTextNode(val));
                            }
                            else {
                                cell.appendChild(document.createTextNode(''));
                            }
                            row.onclick = (function (a) { return function () { options.onSelect(a); } })(data[r]);
                        }
                    }
                    else {
                        var cell = document.createElement('td');
                        cell.appendChild(document.createTextNode(data[r]));
                        row.appendChild(cell);
                        row.onclick = (function (a) { return function () { options.onSelect(a); } })(data[r]);
                    }
                }
            }
        }

        function getDate(ts) {
            var date = new Date(ts.replace(
                /^(\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$/,
                '$4:$5:$6 $2/$3/$1'
            ));
            return date;
        }
    };
    this.OneGridDoc = function (target, options) {
        var tgt = document.getElementById(target);
        var table = document.createElement('table');
        table.className = 'table table-hover table-striped';
        tgt.innerHTML = '';
        tgt.appendChild(table);
        var cc = options.columns.length;
        var header = table.createTHead();
        var headerRow = header.insertRow();
        var body = document.createElement('tbody');

        table.appendChild(body);
        var photoCellHeader = document.createElement('td');
        headerRow.appendChild(photoCellHeader);
        //photoCellHeader.appendChild(document.createTextNode(''));
        if (cc > 0) {
            for (var c = 0; c < cc; c++) {
                var cell = document.createElement('td');
                headerRow.appendChild(cell);
                cell.appendChild(document.createTextNode(options.columns[c].Name));
            }
        }
        // load data from datasource
        this.Load = function load(data) {
            if (data != null) {
                var ct = data.length;
                for (var r = 0; r < ct; r++) {
                    var row = body.insertRow(-1);
                    var photoCell = document.createElement('td');
                    if (data[r].FileCount > 0) {
                        var pLbl = document.createElement('span');
                        pLbl.className = 'label label-success';
                        var pIcon = document.createElement('i');
                        pIcon.className = 'fa fa-camera';
                        pLbl.appendChild(pIcon);
                        photoCell.appendChild(pLbl);
                    }
                    row.appendChild(photoCell);
                    if (cc > 0) {
                        for (var c = 0; c < cc; c++) {
                            var cell, val;
                            cell = document.createElement('td');
                            if (options.columns[c].DataType == 'text') {
                                val = data[r].Fields[options.columns[c].Id];
                            }
                            else if (options.columns[c].DataType == 'date') {
                                val = ods.EDMDateToString(data[r].Fields[options.columns[c].Id]);
                            }
                            else if (options.columns[c].DataType == 'number') {
                                val = new Number(data[r].Fields[options.columns[c].Id]).toLocaleString('en-US');
                            }
                            else {
                                val = data[r].Fields[options.columns[c].Id];
                            }

                            row.appendChild(cell);
                            if (val != null) {
                                cell.appendChild(document.createTextNode(val));
                            }
                            else {
                                cell.appendChild(document.createTextNode(''));
                            }
                            row.onclick = (function (a) { return function () { options.onSelect(a); } })(data[r]);
                        }
                    }
                    else {
                        var cell = document.createElement('td');
                        cell.appendChild(document.createTextNode(data[r]));
                        row.appendChild(cell);
                        row.onclick = (function (a) { return function () { options.onSelect(a); } })(data[r]);
                    }
                }
            }
        }

        function getDate(ts) {
            var date = new Date(ts.replace(
                /^(\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$/,
                '$4:$5:$6 $2/$3/$1'
            ));
            return date;
        }
    };
    this.PendingGrid = function (target, options) {
        var tgt = document.getElementById(target);
        var table = document.createElement('table');
        table.className = 'table table-hover table-striped';
        tgt.innerHTML = '';
        tgt.appendChild(table);
        var header = table.createTHead(),
            headerRow = header.insertRow(),
            body = document.createElement('tbody'),
            docTypeHeader = document.createElement('td'),
            createdOnHeader = document.createElement('td'),
            createdByHeader = document.createElement('td');

        docTypeHeader.appendChild(document.createTextNode('Document Type'));
        createdOnHeader.appendChild(document.createTextNode('Created On'));
        createdByHeader.appendChild(document.createTextNode('Created By'));

        table.appendChild(body);
        headerRow.appendChild(docTypeHeader);
        headerRow.appendChild(createdOnHeader);
        headerRow.appendChild(createdByHeader);

        // load data from datasource
        this.Load = function load(data) {
            if (data != null) {
                var ct = data.length;
                for (var r = 0; r < ct; r++) {
                    var row = body.insertRow(-1),
                        docTypeCell = document.createElement('td'),
                        createdOnCell = document.createElement('td'),
                        createdByCell = document.createElement('td'),
                        docType = ods.App.Schema.DocumentTypes[data[r]['DocumentTypeId']],
                        createdOn = data[r]['CreatedOn'],
                        createdBy = ods.App.Accounts[data[r]['CreatedBy']];

                        if (docType != null) {
                            docTypeCell.appendChild(document.createTextNode(docType.Name));
                        }
                        if (createdOn != null) {
                            createdOnCell.appendChild(document.createTextNode(ods.FormatDate(getDate(createdOn))));
                        }
                        if (createdBy != null) {
                            createdByCell.appendChild(document.createTextNode(createdBy.FirstName + ' ' + createdBy.LastName));
                        }

                    row.appendChild(docTypeCell);
                    row.appendChild(createdOnCell);
                    row.appendChild(createdByCell);

                    row.onclick = (function (a) { return function () { options.onSelect(a); } })(data[r]);
                }
            }
        }

        function getDate(ts) {
            var date = new Date(ts.replace(
                /^(\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$/,
                '$4:$5:$6 $2/$3/$1'
            ));
            return date;
        }
    };
    // 1badge
    this.RenderFile = function (did, cb, fb) {
        var client = this;
        $.ajax({
            url: client.App.APIPath + 'badges/' + did + '/render',
            type: 'POST',
            contentType: 'application/json;charset=utf-8',
            headers: client.App.APIHeaders
        })
        .done(function (uri) { cb(uri); })
        .fail(function (rsp) { client.AlertUser(rsp.status + ' : ' + client.HttpStatusCodes[rsp.status], rsp.statusText, fb); });
    };
}